function y = bpstate_filt(b,a,x,M)
%function y = bpstate_filt(b,a,x,M)
%
% discrete-time block parallel state-space filtering for scalar inputs
% 
% a,b   numerator and denominator of transfer function
% x     input signal vector 
% M     optional block length (optimum is used, if omitted)
% y     output signal vector
%
% G. Doblinger, 12-2000, TU-Wien

if nargin == 0
   help bpstate_filt
   return
end
Ns = max(length(a),length(b))-1;
disp(sprintf('filter order = %d',Ns));
Mopt = round(sqrt(2*Ns));
disp(sprintf('Optimum block length = %d',Mopt));
if nargin == 3
   M = Mopt;
end
Nops = round(Ns/M + 2*Ns + 0.5*(M+1));
disp(sprintf('Operations per sampling interval = %d',Nops));

Nx = length(x);
Nx1 = M * ceil(Nx/M);        
x = [x(:) ; zeros(Nx1-Nx,1)];  % append zeros to make signal length
                               % a multiple of M



[A1,B1,C1,D1] = tf2ss(b,a);    % compute matrices of state-space decomposition
[V,L] = eig(A1);               % compute eigenvalue decomposition
A1 = diag(L);
B1 = inv(V)*B1;
C1 = C1*V;

% compute block state-space matrices

B = zeros(Ns,M);
C = zeros(M,Ns);
Ap = 1;
for n = M-1:-1:1
   Ap = Ap .* A1;
   B(:,n) = Ap.*B1;
   C(M-n+1,:) = C1.*(Ap.');
end
A = Ap .* A1;
B(:,M) = B1;
C(1,:) = C1;
D = toeplitz([D1 ; real(C(1:M-1,:)*B1)],[D1 zeros(1,M-1)]);

ir = find(abs(imag(A)) < 1e-10);
ic = find(imag(A) >= 1e-10);
A = [real(A(ir)) ; A(ic)];
B = [real(B(ir,:)) ; B(ic,:)];
C = [real(C(:,ir)) 2*C(:,ic)];

% perform filter operation

s =  [zeros(size(ir)) ; zeros(size(ic))];
y = zeros(size(x));
for n = 1:M:Nx1
   nb = n:n+M-1;
   y(nb) = real(C*s) + D*x(nb); 
   s = A.*s + B*x(nb);
end
y = y(1:Nx);


