function chirpdft(x,fstart,fstop,L,dBrange)
% function chirpdft(x,fstart,fstop,L,dBrange)
%
% call: chirpdft(x), chirpdft(x,fstart,fstop), chirpdft(x,fstart,fstop,L),
%
% compute and plot DFT of signal using chirp-z algorithm
% (N time points, and L frequency points within arbitrary frequency
% interval)
%
% x         signal vector of length N
% fstart    start frequency of spectrum normalized to sampling
%           frequency fs (default is 0)
% fstop     stop frequency of spectrum normalized to fs (default is 1)
% L         optional number of frequency points (default is L = N) 
% dBrange   optional dynamic range of dB scale (default is 100 dB)
%
% G. Doblinger, TU-Wien, 12-1992

if nargin == 0
   help chirpdft
   return
elseif nargin < 3
   fstart = 0;
   fstop = 1;
end
   
if min(size(x)) ~= 1
   error('CHIRPDFT: x must be a vector')
elseif fstart >= fstop
   error('CHIRPDFT: stop frequency must be greater than start frequency')
elseif fstop > 1
   error('CHIRPDFT: normalized stop frequency must not be greater than 1')
end
N = max(size(x));
if nargin <= 3
   L = N;
   dBrange = 100;
elseif nargin == 4
   if isempty(L), L = N; end
   dBrange = 100;
elseif nargin == 6
   if isempty(dBrange), dBrange = 100; end 
end
w0 = 2*pi*fstart;
fmax = (L-1+fstart)/L;              % maximum possible stop frequency
fstop = min(fmax,fstop);
deltaw = pi/(L-1)*(fstop-fstart);   % (half of) frequency resolution 
n = [0:N-1]';                       
k1 = [-N+1:-1]';
k2 = [0:L-1]';
p_in = exp(-j*(w0*n + deltaw*n.^2));      % input modulation signal
p_out = exp(-j*deltaw*k2.^2);             % output modulation signal 
h = [exp(j*deltaw*k1.^2) ; conj(p_out)];  % impulse response of chirp filter

M = N+L-1;                          % length of convolution result
                                    % (chirp filtering)
Mf = 2^(ceil(log(M)/log(2)));       % find nearest power of 2 for FFT
H = fft(h,Mf);                      % FFT of chirp filter impulse response
X = ifft(fft(x(:).*p_in,Mf) .* H);  % output of chirp filter
                                    % computed via FFT (use FFT length
                                    % Mf, so cyclic convolution result
                                    % is equal to linear convolution
                                    % result)

X = X(N:M) .* p_out;                % perform modulation of output
                                    % to complete computation of DFT

X1 = ifft(fft(n.*x(:).*p_in,Mf) .* H);    % use input n*x(n) for
                                    % computation of group delay  
                                    % without differentiation,
                                    % since X1 = j*dX/dw, and
                                    % tg = -imag(d(ln(X)/dw) = real(X1/X)
X1 = X1(N:M) .* p_out;

if L < 40
   f = 0:L-1;                       % frequency indices
   str_plot = 'discret';
   str_freq = 'Frequency index (0=fstart)';
else
   f = linspace(fstart,fstop,L);    % frequencies normalized to fs
   str_plot =  'plot';
   str_freq = 'Normalized frequency';
end             

Xlog = log10(X + eps);              % take complex logarithm
Xmag = 20*real(Xlog);               % compute magnitude in dB

magmax = max(Xmag);
magmin = min(Xmag);
if magmax - magmin < 1e-10          % avoid log rounding errors
                                    % (set resolution to 1e-10 dB) 
   Xmag = zeros(size(X));
end
dBrange = abs(dBrange);
if magmax - magmin > dBrange        % eliminate spectral nulls
                                    % (set dynamic range to dBrange dB)
   imin = find(Xmag < magmax-dBrange);
   Xmag(imin) = (magmax-dBrange)*ones(length(imin),1);
end

tgflag = 0;
Xr = real(X);  Xi = imag(X);
if abs(Xr) < 1e-10                  % real part approx. zero
   Xr = zeros(size(X));
   ph = zeros(size(X));
   ip = find(Xi > 1e-10);
   in = find(Xi < -1e-10);
   if ~isempty(ip), ph(ip) = pi/2 * ones(length(ip),1); end
   if ~isempty(in), ph(in) = -pi/2 * ones(length(in),1); end
   tg = zeros(size(X));
elseif abs(Xi) < 1e-10              % imaginary part approx. zero
   Xi = zeros(size(X));
   ph = zeros(size(X));
   tg = zeros(size(X));
else
   ph = 1/log10(exp(1))*imag(Xlog);    % compute phase
   tg = real(X1 ./ (X + 1e-15));       % group delay
   inan = find(abs(tg) >= 1e3);        % eliminate isolated group delay spikes
   if ~isempty(inan)                   % interpolate at spike positions
      Ninan = length(inan);
      tg(inan) = 0.5*(tg(max(1,inan-1))+tg(min(Ninan,inan+1)));
      if Ninan > 1 & inan(Ninan) == Ninan
         tg(Ninan) = tg(Ninan-1);
      end
      if inan(1) == 1, tg(1) = tg(2); end
   end
   if abs(max(tg) - min(tg)) < 1e-3    % nearly constant group delay
      tg = max(tg)*ones(size(tg));
   end
   inan = find(abs(tg) >= 1e10);       % remaining group delay spikes
   if ~isempty(inan)
      tgflag = 1;
      disp('CHIRPDFT INFO: Phase/Group delay computation failed');
   else
      tgflag = 0;
   end
end

% section to plot results (comb-like plot for Nf < 40)

close all
figurebackcolor = 'black';

if strcmp(upper(computer),'PCWIN')
   pos1 = [0.5 0.51 0.49 0.405];
   axfnt = 9;
   ticfnt = 8;
   yoff = 0.49;
else	
   pos1 = [0.5 0.6 0.49 0.38];
   axfnt = 13;
   ticfnt = 12;
   yoff = 0.51;
   
end
fmag = figure('numbertitle','off','name','Frequency domain: Chirp DFT magnitude (x-zoom on)',...
   'Units','normal','Position',pos1);
colordef(fmag,figurebackcolor);

subplot(2,1,1), eval([str_plot,'(f,abs(X))']);
if L >= 40
   grid on;
end
h1 = xlabel(str_freq);
h2 = ylabel('Magnitude');
set(h1,'fontsize',axfnt);
set(h2,'fontsize',axfnt);
set(gca,'fontsize',ticfnt);
subplot(2,1,2), eval([str_plot,'(f,Xmag)']);
if L >= 40
   grid on;
end
h2 = ylabel('Magnitude in dB');
set(h2,'fontsize',axfnt);
set(gca,'fontsize',ticfnt);
zoom(fmag,'xon');

pos1(1) = pos1(1) - 0.495;  % position of next figure
if ~tgflag
   fph = figure('numbertitle','off','name',...
		'Frequency domain: Chirp DFT phase, and group delay (x-zoom on)',...
		'Units','normal','Position',pos1);
   colordef(fph,figurebackcolor);

   subplot(2,1,1), eval([str_plot,'(f,ph)']);
   if L >= 40
      grid on;
   end
   h1 = xlabel(str_freq);
   h2 = ylabel('DFT phase (rad)');
   set(h1,'fontsize',axfnt);
   set(h2,'fontsize',axfnt);
   set(gca,'fontsize',ticfnt);
   subplot(2,1,2), eval([str_plot,'(f,tg)']);
   if L >= 40
      grid on;
   end
   h2 = ylabel('Group delay (samples)');
   set(h2,'fontsize',axfnt);
   set(gca,'fontsize',ticfnt);
   zoom(fph,'xon');
   pos1(2) = pos1(2) - yoff;  % position of next figure
end

figure('numbertitle','off','name','Complex domain','Units','normal','Position',pos1);

xmax = ceil(max([max(abs(Xr)) max(abs(Xi))]));
subplot(1,1,1), plot(Xr, Xi, Xr(1), Xi(1), 'og');
set(gca,'fontsize',ticfnt);
axis([-xmax xmax -xmax xmax]);
axis('square');
h1 = title('DFT locus diagram (o ... start freq.)');
h2 = xlabel('DFT real part');
h3 = ylabel('DFT imaginary part');
set(h1,'fontsize',axfnt);
set(h2,'fontsize',axfnt);
set(h3,'fontsize',axfnt);
grid on

