function [y,ostate] = oladd(x,h,L,istate)
%function [y,ostate] = oladd(x,h,L,istate)
%
% overlap-add method for FIR filtering
%
% x      input signal vector
% h      impulse response vector of length N
% L      FFT length (size of overlapping signal blocks)
%        L > N = length(h) 
% istate optional initial state vector of length N-1
%        (a zero state is used, if omitted)
% y      output signal vector
% ostate optional final state vector of length N-1
%
% G. Doblinger, TU-Wien, 12-1999

if nargin == 0
   help olsave
   return
end

[xr,xc] = size(x);
if min(xr,xc) > 1
   error('x must be a vector');
end
x = x(:);
h = h(:);
Nx = max(xr,xc);
N = length(h);

if Nx <= N                     % no block processing
   if nargin == 3
      istate = zeros(N-1,1);
   else
      if length(istate) ~= N-1
	 error('state vector must have length N-1');
      end
   end   
   [y,ostate] = filter(h,1,x,istate);
   return
end

if L < N
   error('Nf less than length(h)');
end
ostate = x(Nx-N+2:Nx);         % final state vector

M = L - N + 1;                 % block hop size
H = fft(h,L);                  % filter transfer function
if nargin == 3                 % include state vector
   x = [zeros(N-1,1) ; x];
else
   if length(istate) ~= N-1
      error('state vector must have length N-1');
   end
   x = [istate(:) ; x];
end
Nx1 = Nx + N - 1;     
Nx2 = M*ceil(Nx1/M);           % make length of x a multiple of M 
x = [x ; zeros(Nx2-Nx1,1)];    % pad with zeros, if necessary
y = zeros(Nx2+N,1);

% block processing loop 

for n = 1:M:Nx2-M+1
   y1 = ifft(fft(x(n:n+M-1),L).*H);
   n1 = n:n+L-1;
   y(n1) = y(n1) + y1;
end

y = y(N:Nx1);
if abs(imag(x)) < eps
   y = real(y);
end
if xc > xr
   y = y.';
end

