function [Ha,Hs,Nv]=nufbark4(h96,h64,h16_12)
% function [Ha,Hs,Nv]=nufbark4(h96,h64,h16_12)
%
% bark scale filter bank design using qmfuni.m (unicmfb.m)
% and interpolation/decimation for nonuniform prototype filters
% (version with tight approximation of critical band analysis, 25 channels)
%
% h96...    odd length prototype low pass filter impulse responses
% Ha        analysis filter impulse responses (as colunm vectors)
% Hs        synthesis filter impulse responses (as colunm vectors)
% Nv        vector of decimation factors
%
% G. Doblinger, TU-Wien, 08-1996

% requires function interpo4.m

if nargin == 0
   help nufbark4
   return
end

L = length(h96);
if ~rem(L,2)
   error('filter h96 must have odd length');
elseif ~rem(L+1,8)
   error('L96+1 must be divisible by 8');
end
L64 = length(h64);
if ~rem(L64,2)
   error('filter h64 must have odd length');
elseif ~rem(L64+1,8)
   error('L64+1 must be divisible by 8');
end
L16_12 = length(h16_12);
if ~rem(L16_12,2)
   error('filter h16_12 must have odd length');
end

% find highest order transition filter by interpolating
% h16_12 by a factor 4

h64_48 = interpo4(h16_12);
L64_48 = length(h64_48);
h64_48 = sqrt(48)*h64_48/sum(h64_48);

% form other impulse responses by decimation
% zero-pad impulse responses to equal length L

h48 = sqrt(2)*h96(1:2:L);
L1 = (L+1)/2;
h48 = [zeros(round((L-L1)/2),1);h48;zeros(round((L-L1)/2),1)];
h24 = 2*h96(1:4:L);
L1 = (L+1)/4;
h24 = [zeros(round((L-L1)/2),1);h24;zeros(round((L-L1)/2),1)];
h12 = sqrt(8)*h96(1:8:L);
L1 = (L+1)/8;
h12 = [zeros(round((L-L1)/2),1);h12;zeros(round((L-L1)/2),1)];

h32 = sqrt(2)*h64(1:2:L64);
L1 = (L64+1)/2;
h32 = [zeros(round((L-L1)/2),1);h32;zeros(round((L-L1)/2),1)];
h16 = 2*h64(1:4:L64);
L1 = (L64+1)/4;
h16 = [zeros(round((L-L1)/2),1);h16;zeros(round((L-L1)/2),1)];
h8 = sqrt(8)*h64(1:8:L64);
L1 = (L64+1)/8;
h8 = [zeros(round((L-L1)/2),1);h8;zeros(round((L-L1)/2),1)];

h32_24 = sqrt(2)*h64_48(1:2:L64_48);
L1 = (L64_48+1)/2;
h32_24 = [zeros(round((L-L1)/2),1);h32_24;zeros(round((L-L1)/2),1)];
h16_12 = 2*h64_48(1:4:L64_48);
L1 = (L64_48+1)/4;
h16_12 = [zeros(round((L-L1)/2),1);h16_12;zeros(round((L-L1)/2),1)];

h64 = [zeros((L-L64)/2,1);h64;zeros((L-L64)/2,1)];
h64_48 = [zeros((L-L64_48)/2,1);h64_48;zeros((L-L64_48)/2,1)];

% determine frequency spacings, build impulse responses of channel filters

N96 = 6;
N64 = 4;
N48 = 3;
N32 = 2;
N24 = 3;
N16 = 2;
N12 = 3;
N8 = 2;
Nv = [96*ones(1,N96) 64*ones(1,N64) 48*ones(1,N48) 32*ones(1,N32) ...
      24*ones(1,N24) 16*ones(1,N16) 12*ones(1,N12) 8*ones(1,N8)];  % subsampling factors
N = length(Nv);
Ha = zeros(L,N);
Hs = zeros(L,N);
N0 = (L-1)/2;
n = [0:L-1]'-N0;

% compute impulse responses of first channels

for k = 1:N96
   Nk = Nv(k);
   theta = pi/(2*Nk)*(2*k-1);
   nn = n - Nk/2;
   Ha(:,k) = h96 .* cos(theta*nn);
   nn = n + Nk/2;
   Hs(:,k) = h96 .* cos(theta*nn);
end
k0 = k+1;
kv = [4,6,7,6,6,7,6,6,7,6];               % vector of modulation start indices
Nb = [N64,1,N48-1,N32,1,N24-1,N16,1,N12-1,N8];
Hb = [h64,h64_48,h48,h32,h32_24,h24,h16,h16_12,h12,h8];

% compute remaining channel impulse responses

for m = 1:length(kv)
   k1 = kv(m);
   for k = k0:k0+Nb(m)-1
      Nk = Nv(k);
      k1 = k1 + 1;
      phi = pi/(2*Nk)*(2*k1-1)*n;
      phi0 = (2*k-1)*pi/4;
      hpt = Hb(:,m);
      if any(imag(hpt))
         Ha(:,k) = real(hpt) .* cos(phi-phi0) - imag(hpt) .* sin(phi-phi0);
         Hs(:,k) = real(hpt) .* cos(phi+phi0) - imag(hpt) .* sin(phi+phi0);
      else
         Ha(:,k) = hpt .* cos(phi-phi0);
         Hs(:,k) = hpt .* cos(phi+phi0);
      end
   end
   k0 = k+1;
end

% compute reconstruction error

Nx = 2*L;
x = [1 zeros(1,Nx-1)];
y = zeros(1,Nx);
for k = 1:N
   ya = filter(2*Ha(:,k),1,x);
   m = 1:Nv(k):Nx;
   u = zeros(1,Nx);
   u(m) = ya(m);
   y = y + filter(2*Hs(:,k),1,u);
end

% compute and plot transfer function

if L < 150
   Nft = 512;
else
   Nft = 1024;
end
f = linspace(0,1,Nft);

magH = fft(Ha,2*Nft);
magH = abs(magH(1:Nft,:));
magH = magH/max(max(magH));
eps1 = 1e-6;
i = find(magH <= eps1);
if ~isempty(i)
   magH(i) = eps1 * ones(size(i));
end
magY = abs(freqz(y,1,pi*f));

if strcmp(upper(computer),'PCWIN')
   pos = [0.5 0.5 0.49 0.415];
   axfnt = 8;
   ticfnt = 7;
else	
   pos = [0.5 0.6 0.49 0.4];
   axfnt = 12;
   ticfnt = 10;
end
pos(1) = pos(1) - 0.495;
fdom = figure('name','Frequency domain (xy-zoom on)','Units','normal','Position',pos);
figurebackcolor = 'black';
colordef(fdom,figurebackcolor);

xstr = 'f normalized to Fsample/2';
ystr = 'Magnitude in dB';
tstr1 = ['Modulated filter bank transfer functions, N = ',int2str(N),', L = ',int2str(L)];
tstr2 = 'Overall transfer function of analysis+synthesis filter bank';
subplot(2,1,1), plot(f,20*log10(magH));
gh1 = xlabel(xstr); set(gh1,'fontsize',axfnt);
gh1 = ylabel(ystr); set(gh1,'fontsize',axfnt);
gh1 =  title(tstr1); set(gh1,'fontsize',axfnt); grid on;
set(gca,'fontsize',ticfnt);
subplot(2,1,2), plot(f,20*log10(magY(:)));
gh1 = xlabel(xstr); set(gh1,'fontsize',axfnt);
gh1 = ylabel(ystr); set(gh1,'fontsize',axfnt);
gh1 =  title(tstr2); set(gh1,'fontsize',axfnt); grid on;
set(gca,'fontsize',ticfnt);
zoom(fdom,'on');

