function [y,Y] = dtwt(h,N,x)
% function dtwt(h,N,x)
% DTWT using polyphase 2-channel QMF filter banks
% for linear phase filters, and lattice filters for
% orthonormal wavelet filters
%
% h    prototype low-pass filter response (or lattice filter
%      coefficients)
% N    number of channels 
% x    input signal vector (length > 2^N+length(h))
%      (periodic noise is used, is omitted)
% y    output of DTWT + IDTWT (delay compensated)
% Y    Matrix of DTWT subband signals (as column vectors)
%
% G. Doblinger, TU-Wien, 2-2001

if nargin == 0
   help dtwt
   return
elseif nargin == 2
   xflag = 1;
else
   xflag = 0; 
end

h = h(:);
L  = length(h);
Nd =  sum([0:L-1]'.*(h.^2))/sum(h.^2);   % energy delay
if abs(Nd - (L-1)/2) < 1e-3
   disp('using polyphase filters');
   htype = 1;
else
   disp('using lattice filters'); 
   htype = 0;
end
N = max(2,N);

% TP and HP filter impulse responses

if htype == 1
   h1 = h;
   h2 = h1;
   h2(2:2:end) = -h2(2:2:end);           % h2(n) = (-1)^n * h1(n)
   p1 = h(1:2:end);                      % polyphase filters 
   p2 = h(2:2:end);   
else
   [h1,h2] = latafib2(h,[1;zeros(2*L-1,1)]);
   L = length(h1);
end

% select input signal to compute overall filter bank response

if xflag == 1
   Nx = max(2048,2^(N+2));
   Nx2 = Nx/2;
   %x = [1 zeros(1,Nx-1)];
   x = perinois(Nx,Nx2,[],sqrt(2));     % create periodic white noise
   x = x(:);
else
   Nx = length(x);
   if Nx < 2^N + L
      error('length of x must be greater than 2^N + length(h)');
   end
   Nx2 = Nx;
   x = x(:);
end

% analysis filter bank output signals stored in matrix Y

Y = zeros(Nx2+L,N);
yl = x;
if htype == 1
   for n = 1:N-1
      [yl,yh] = polyafib2(p1,p2,yl);
      Y(1:length(yh),n) = yh;
   end
else
   for n = 1:N-1
      [yl,yh] = QMFaLattice(h,yl);
      Y(1:length(yh),n) = yh;
   end
end
Y(1:length(yl),N) = yl;

% synthesis filter bank

y = yl;
if htype == 1
   for n = N-1:-1:1
      y = polysfib2(p1,p2,y,Y(1:length(y),n));
   end
else
   for n = N-1:-1:1
      y = QMFsLattice(h,y,Y(1:length(y),n));
   end
end
if xflag == 1
   %y = y(1:Nx2);
   y = y(Nx2/2+1:Nx2/2+Nx2);            % take a period to avoid filter settling time
else
   y = y(1:Nx2);
   return
end

% compute and plot transfer function

f = linspace(0,1,Nx2);
magH1 = abs(freqz(h1,1,pi*f));
magH1 = magH1/max(magH1);
eps1 = 1e-6;
i = find(magH1 <= eps1);
if ~isempty(i)
   magH1(i) = eps1 * ones(size(i));
end
magH2 = abs(freqz(h2,1,pi*f));
magH2 = magH2/max(magH2);
i = find(magH2 <= eps1);
if ~isempty(i)
   magH2(i) = eps1 * ones(size(i));
end
magY = fft(y);
magY = abs(magY(1:Nx2/2));
fy = linspace(0,1,Nx2/2);

i = find(magY <= eps1);
if ~isempty(i)
   magY(i) = eps1 * ones(size(i));
end

close all
if strcmp(upper(computer),'PCWIN')
   pos = [0.5 0.5 0.49 0.415];
   axfnt = 8;
   ticfnt = 7;
else	
   pos = [0.5 0.6 0.49 0.4];
   axfnt = 12;
   ticfnt = 10;
end

figure('name','Time domain','Units','normal','Position',pos);
xstr = 'Time index n';
n = [0:L-1];
Nd_lp = (L-1)/2;
Nd_hp = L - Nd_lp - 1;
ystr1 = 'h_{lp}(n)';
ystr2 = 'h_{hp}(n)';
tstr1 = ['Analysis low pass filter, N_{lp} = ',int2str(L)];
tstr2 = ['Analysis high pass filter, N_{hp} = ',int2str(L)];
subplot(2,1,1), discret(n,h1);
h1 = ylabel(ystr1); set(h1,'fontsize',axfnt);
h1 = title(tstr1); set(h1,'fontsize',axfnt);
set(gca,'fontsize',ticfnt);
subplot(2,1,2), discret(n,h2);
h1 = xlabel(xstr); set(h1,'fontsize',axfnt);
h1 = ylabel(ystr2); set(h1,'fontsize',axfnt);
h1 = title(tstr2); set(h1,'fontsize',axfnt);
set(gca,'fontsize',ticfnt);

pos(1) = pos(1) - 0.495;  % position of next figure
ftf = figure('name','Frequency domain (y-zoom on)','Units','normal','Position',pos);

xstr = 'f normalized to Fsample/2';
ystr = 'Magnitude in dB';
if htype == 1
   tstr1 = 'QMF-bank transfer functions (polyphase filters)';
else
   tstr1 = 'QMF-bank transfer functions (lattice filters)';
end
tstr2 = ['DTWT + IDTWT transfer function, N = ',...
         int2str(N), ', periodic noise input'];
subplot(2,1,1), plot(f,20*log10(magH1),'y',f,20*log10(magH2),'m');
h1 = ylabel(ystr); set(h1,'fontsize',axfnt);
h1 = title(tstr1); set(h1,'fontsize',axfnt); grid on
set(gca,'fontsize',ticfnt);
subplot(2,1,2), plot(fy,20*log10(magY),'y');
h1 = xlabel(xstr); set(h1,'fontsize',axfnt);
h1 = ylabel(ystr); set(h1,'fontsize',axfnt);
h1 = title(tstr2); set(h1,'fontsize',axfnt); grid on
set(gca,'fontsize',ticfnt);
zoom(ftf,'yon');

% --------------------------------------------------------------------

function [yl,yh] = polyafib2(p1,p2,x)

% compute analysis filter bank (polyphase filters)

u1 = conv(p1,x(1:2:end));
u2 = conv(p2,[0 ; x(2:2:end-1)]);
yl = u1 + u2;
yh = u1 - u2;

% --------------------------------------------------------------------

function y = polysfib2(p1,p2,yl,yh)

% compute synthesis filter bank (polyphase filters)

L = 2*length(p1);
Ly = 2*length(yl)+L-2;
v1 = yl - yh;
v2 = yl + yh;
y = zeros(Ly,1);
y(1:2:Ly) = conv(p1,v1);
y(2:2:Ly) = conv(p2,v2); 
y = y(L:Ly+3-L);   % compensate delay of 2 channel analysis/synthesis filter bank

% --------------------------------------------------------------------

function [yl,yh] = Fpolyafib2(p1,p2,x)

% compute analysis filter bank (polyphase filters)
% (version using filter function)

Nx = length(x);
N = length(p1);
Ny = floor((Nx-1)/2+N);   % required output signal length

% compute decimated lattice input signals

p = x(1:2:Nx);
q = x(2:2:Nx-1);
p = [p(:) ; zeros(Ny-length(p),1)];        % zero padding
q = [0 ; q(:) ; zeros(Ny-length(q)-1,1)];  % delay + zero padding

u1 = filter(p1,1,p);
u2 = filter(p2,1,q);
yl = u1 + u2;
yh = u1 - u2;

% --------------------------------------------------------------------

function y = Fpolysfib2(p1,p2,xl,xh)

% compute synthesis filter bank (polyphase filters)
% (version using filter function)

N = length(p1);
Nx = length(xl) + N;
xl = [xl(:) ; zeros(N,1)];
xh = [xh(:) ; zeros(N,1)];
v1 = xl - xh;
v2 = xl + xh;
p = filter(p1,1,v1);
q = filter(p2,1,v2);

% up-sampling commutator

Nx = 2*Nx;
y = zeros(1,Nx);
y(1:2:Nx) = p;
y(2:2:Nx) = q;
y = y(2*N:Nx-2*N+1);   % compensate filter gain and delay

% --------------------------------------------------------------------

function [yl,yh] = QMFaLattice(alpha,x)
%
% compute response of decimated 2-channel QMF lattice analysis filter bank
%
% alpha    vector of lattice coefficients computed with Fir2Lattice()
% x        input signal vector
% yl, yh   output signal vectors of low pass and high pass filters

% Db, 6.1.98
% ref.: P.P.Vaidyanathan, "Lattice structures for optimal design and robust
%     implementation of two-channel perfect-reconstruction QMF banks", IEEE, Trans.
%     ASSP, Vol. 36, pp.81-94, Jan. 1988.

Nx = length(x);
N = length(alpha);
Ny = floor((Nx-1)/2+N); % required output signal length

% compute decimated lattice input signals

q0 = x(1:2:Nx-1);       % down-sampling commutator
p0 = x(2:2:Nx);
zpad = zeros(Ny-length(p0),1);
p0 = [p0(:) ; zpad];    % zero padding
q0 = [q0(:) ; zpad];
Nq = length(q0)-1;

% compute first stage separately

p = p0 - alpha(1)*q0;
q = q0 + alpha(1)*p0;

for n = 2:N
   q = [0 ; q(1:Nq)];   % delayed q-vector
   pt = p;
   p = p - alpha(n)*q;
   q = q + alpha(n)*pt;
end

beta = 1/sqrt(prod(1+alpha.^2));
yl = beta*p;
yh = beta*q;

% --------------------------------------------------------------------

function [yl,yh] = latafib2(alpha,x)
%
% compute response of non-decimated 2-channel QMF lattice analysis filter bank
%
% alpha    vector of lattice coefficients computed with Fir2Lattice()
% x        input signal vector
% yl, yh   output signal vectors of low pass and high pass filters

Nx = length(x);
N = length(alpha);

% compute lattice input signals (no decimation)

p0 = x(:);
q0 = [0 ; p0(1:Nx-1)];
Nq = Nx - 2;

% compute first stage separately

p = p0 - alpha(1)*q0;
q = q0 + alpha(1)*p0;

for n = 2:N
   q = [0 ; 0 ; q(1:Nq)];   % delayed q-vector
   pt = p;
   p = p - alpha(n)*q;
   q = q + alpha(n)*pt;
end

beta = 1/sqrt(2*prod(1+alpha.^2));
yl = beta*p;
yh = beta*q;


% --------------------------------------------------------------------

function y = QMFsLattice(alpha,xl,xh)
%
% compute response of 2-channel QMF lattice synthesis filter bank
%
% alpha    vector of lattice coefficients computed with Fir2Lattice()
% xl, xh   input signal vectors computed with QMFaLattice()
% y        output signal vector

% Db, 6.1.98
% ref.: P.P.Vaidyanathan, P.Hoang, "Lattice structures for optimal design and robust
%     implementation of two-channel perfect-reconstruction QMF banks", IEEE, Trans.
%     ASSP, Vol. 36, pp.81-94, Jan. 1988.

N = length(alpha);
Nx = length(xl) + N;
Nq = Nx - 1;
alpha = alpha(N:-1:1);  % use reversed coefficients

% compute first stage separately

xl = [xl(:) ; zeros(N,1)];
xh = [xh(:) ; zeros(N,1)];
p = xl + alpha(1)*xh;
q = xh - alpha(1)*xl;

for n = 2:N
   p = [0 ; p(1:Nq)];   % delayed p-vector
   pt = p;
   p = p + alpha(n)*q;
   q = q - alpha(n)*pt;
end

% up-sampling commutator

Nx = 2*Nx;
y = zeros(1,Nx);
y(1:2:Nx) = q;
y(2:2:Nx) = p;
y = y(2*N-1:Nx-2*N)/sqrt(prod(1+alpha.^2));   % compensate filter gain and delay
