function ht = nuftrans(dp,h1,N1,h2,N2,Lt,plt)
% function ht = nuftrans(dp,h1,N1,h2,N2,Lt,plt)
%
% transition band pass filter design for nonuniform filterbanks
%
% dp        design procedure, 'o' = optimization, 'l' = linear equation solving
% h1        prototype filter impulse response at lower frequencies
% N1        spacing of uniform part at lower frequencies
% h2        prototype filter impulse response at higher frequencies
% N2        spacing of uniform part at higher frequencies
% Lt        length of transition filter
% plt       1 to enable graphics output, 0 otherwise
% ht        prototype impulse response of transition filter

% (c) copyright 8-1996, G.Doblinger, Th. Zeitlhofer, TU-Wien

if nargin == 0
   help nuftrans
   return
end

start = clock;

fc1 = 1/(2*N1);
fc2 = 1/(2*N2);

h1 = h1(:)/sum(h1);
h2 = h2(:)/sum(h2);
L1 = length(h1);
L2 = length(h2);
Lv = [L1,L2,Lt];
evenodd = sum(rem(Lv,2));
if evenodd ~= 0 & evenodd ~= 3
   error('Impulse response lengths must all be even or all be odd');
end
L = max(Lv);               % common length of all filters
Lh = floor((L+1)/2);
N0 = (L-1)/2;              % delay of prototype filters
Nz1 = (L-L1)/2;            % number of zeros appended at both sides of impulse response h1
Nz2 = (L-L2)/2;
Nzt = (L-Lt)/2;

% compute constant vectors and matrices for error vector

Lfp = round(L/2);          % no. of passband freq. points
Lfs = L;                   % no. of stopband freq. points
pifc1 = pi*fc1;            % pi/2N1
pifc2 = pi*fc2;            % pi/2N2
n = [Nz1:Lh-1] - N0;
theta = pifc1*linspace(0,2,Lfp).';
C1 = 2*cos(theta*n);
n = [Nz2:Lh-1] - N0;
theta = pifc2*linspace(0,2,Lfp).';
C2 = 2*cos(theta*n);
n = [Nzt:Lh-1] - N0;
theta = linspace(-pifc2-pifc1,-pifc2+pifc1,Lfp).';
phi = theta*n;
Ut1 = 2*[cos(phi) sin(phi)];
theta = pifc2*linspace(0,2,Lfp).';
phi = theta*n;
Ut2 = 2*[cos(phi) sin(phi)];
theta = pi*linspace(2*fc2,2-fc2-fc1,Lfs).';
phi = theta*n;
Uts = 2*[cos(phi) sin(phi)];
theta = linspace(-pifc2+pifc1,0,Lfp).';
phi = theta*n;
Utp = 2*[cos(phi) sin(phi)];

L1h = floor((L1+1)/2);
L2h = floor((L2+1)/2);
Lth = floor((Lt+1)/2);
if rem(L,2)
   C1(:,L1h) = ones(Lfp,1);
   C2(:,L2h) = ones(Lfp,1);
   Ut1(:,Lth) = ones(Lfp,1);
   Ut2(:,Lth) = ones(Lfp,1);
   Uts(:,Lth) = ones(Lfs,1);
   Utp(:,Lth) = ones(Lfp,1);
   imax = 2*Lth-1;
   Ut1 = Ut1(:,1:imax);
   Ut2 = Ut2(:,1:imax);
   Uts = Uts(:,1:imax);
   Utp = Utp(:,1:imax);
end

h1 = h1(1:L1h);
h2 = h2(1:L2h);
G1 = C1*h1;
G2 = C2*h2;
a1 = G1.*G1(Lfp:-1:1);
a2 = G2.*G2(Lfp:-1:1);
a3 = 1 - G1.^2;
a4 = 1 - G2(Lfp:-1:1).^2;
a5 = ones(Lfp,1);

% create starting solution

ht = [zeros((Lt-L2)/2,1);h2];
if rem(L,2)
   ht = [ht(1:Lth);zeros(Lth-1,1)];
else
   ht = [ht(1:Lth);zeros(Lth,1)];
end

Q = Uts'*Uts;     % matrix to compute stop band energy

clear theta phi n C1 C2 G1 G2 h1 h2 Uts

% find optimum transition filter (low pass filter)

if strcmp(upper(dp),'L')
   alpha1 = 5;
   alpha2 = 5;
   alpha3 = 1;
   alpha4 = 1;
   alpha5 = 1;
   alphas = 100;
   epsil = 1e-6;
   tau = 0.7;
   tau1 = 1-tau;
   Niter = 200;
   I = ones(1,length(ht));
   for k = 1:Niter
      d = Ut1*ht;
      B1 = (d(Lfp:-1:1)*I).*Ut1;
      B3 = (d*I).*Ut1;
      d = Ut2*ht;
      B2 = (d(Lfp:-1:1)*I).*Ut2;
      B4 = (d*I).*Ut2;
      B5 = ((Utp*ht)*I).*Utp;
      U = alpha1*B1'*B1+alpha2*B2'*B2+alpha3*B3'*B3+alpha4*B4'*B4+alpha5*B5'*B5+alphas*Q;
      v = alpha1*B1'*a1+alpha2*B2'*a2+alpha3*B3'*a3+alpha4*B4'*a4+alpha5*B5'*a5;
      gt = U \ v;                               % solution for transition low pass filter
      ht = tau1*ht + tau*gt;                    % impulse response update (real part)
      err = norm(ht-gt)/norm(ht);               % impulse response error
      disp(['Iteration ', int2str(k), ', Error = ', num2str(err)]);
      if err < epsil, break, end
   end

elseif strcmp(upper(dp),'O')                    % solve nonlinear optimization problem

   Lto = length(ht);
   Niter = 200;
   alpha = [5 5 1 1 1 100];
   A = [a1 a2 a3 a4 a5];
   vs = version;
   if strcmp(vs(1),'5')
      options(1) = 1;                           % turn on display
      options(2) = 5e-5;                        % termination criterion for variable h
      options(3) = 5e-5;                        % termination criterion for cost func. (f)
      options(6) = 1;                           % DFP-algorithm for fminu()
      options(14) = Niter;                      % maximum number of iterations
      % options(9) = 1;                         % check gradient
      ht = fminu('nuf_err',ht,options,'nuf_grad',Lfp,Lto,A,Ut1,Ut2,Utp,Q,alpha);
   else                                         % new optimization toolbox with MATLAB 6.0
      options = optimset('Display','on','TolX',5e-5,'TolFun',5e-5,...
         'MaxIter',Niter,'LargeScale','off','GradObj','on');
      ht = fminunc('nuf_err_grad',ht,options,Lfp,Lto,A,Ut1,Ut2,Utp,Q,alpha);
   end

else
   error('no such optimization procedure');
end

% form symmetric prototype low pass filter impulse response

j = sqrt(-1);
htr = ht(1:Lth);
if rem(L,2)
   hti = ht(Lth+1:2*Lth-1);
   ht = sqrt(N2)*([htr ; htr(Lth-1:-1:1)] +j*[hti ; 0 ; -hti(Lth-1:-1:1)]);
else
   hti = ht(Lth+1:2*Lth);
   ht = sqrt(N2)*([htr ; htr(Lth:-1:1)] +j*[hti ; -hti(Lth:-1:1)]);
end

disp(sprintf('\n Elapsed time = %f sec.', etime(clock,start)));

% compute and plot transfer function

if plt
   Nft = 512;
   f = linspace(0,1,Nft);

   magH = abs(fft(ht,Nft));
   magH = magH/max(max(magH));
   eps1 = 1e-6;
   i = find(magH <= eps1);
   if ~isempty(i)
      magH(i) = eps1 * ones(size(i));
   end
   close all
   if strcmp(upper(computer),'PCWIN')
      pos = [0.5 0.5 0.49 0.415];
      axfnt = 8;
      ticfnt = 7;
   else	
      pos = [0.5 0.6 0.49 0.4];
      axfnt = 12;
      ticfnt = 10;
   end
   fdom = figure('name','Frequency domain (xy-zoom on)','Units','normal','Position',pos);
   figurebackcolor = 'black';
   colordef(fdom,figurebackcolor);

   xstr = 'f normalized to Fsample';
   ystr = 'Magnitude in dB';
   tstr = ['Transition prototype filter transfer function, Lt = ',int2str(Lt)];
   plot(f,20*log10(magH))
   gh1 = xlabel(xstr); set(gh1,'fontsize',axfnt);
   gh1 = ylabel(ystr); set(gh1,'fontsize',axfnt);
   gh1 =  title(tstr); set(gh1,'fontsize',axfnt); grid on;
   set(gca,'fontsize',ticfnt);
   grid on;
   zoom(fdom,'on');
end