function polyfib2(h)
% function polyfib2(h)
% compute polyphase 2-channel QNF filter bank
% (for linear phase filters only)
%
% h    prototype low-pass filter response
%
% G. Doblinger, TU-Wien, 1-2001

if nargin == 0
   help polyfib2
   return
end

% compute polyphase filters

p1 = h(1:2:end);
p2 = h(2:2:end);

% compute overall filter bank response

Nx = 1024;
Nx2 = Nx/2;
x = perinois(Nx,Nx2,[],sqrt(2));     % create periodic white noise
x = x(:);

% analysis filter bank output signals y1, y2

u1 = filter(p1,1,x(1:2:end));
u2 = filter(p2,1,[0 ; x(2:2:end-1)]);
y1 = u1 + u2;
y2 = u1 - u2;

% synthesis filter bank

v1 = y1 - y2;
v2 = y1 + y2;
y = zeros(size(x));
y(1:2:end) = filter(p1,1,v1);
y(2:2:end) = filter(p2,1,v2); 
y = y(Nx2+1:Nx);                     % take second period to avoid filter settling time

% compute and plot transfer function

f = linspace(0,1,Nx2);
N  = length(h);
h1 = h(:);
h2 = h1;
h2(2:2:N) = -h2(2:2:N);              % h2(n) = (-1)^n * h1(n)
magH1 = abs(freqz(h1,1,pi*f));
magH1 = magH1/max(magH1);
eps1 = 1e-6;
i = find(magH1 <= eps1);
if ~isempty(i)
   magH1(i) = eps1 * ones(size(i));
end
magH2 = abs(freqz(h2,1,pi*f));
magH2 = magH2/max(magH2);
i = find(magH2 <= eps1);
if ~isempty(i)
   magH2(i) = eps1 * ones(size(i));
end
magY = fft(y);
magY = abs(magY(1:Nx2/2));
fy = linspace(0,1,Nx2/2);

i = find(magY <= eps1);
if ~isempty(i)
   magY(i) = eps1 * ones(size(i));
end

close all
if strcmp(upper(computer),'PCWIN')
   pos = [0.5 0.5 0.49 0.415];
   axfnt = 8;
   ticfnt = 7;
else	
   pos = [0.5 0.6 0.49 0.4];
   axfnt = 12;
   ticfnt = 10;
end

figure('name','Time domain','Units','normal','Position',pos);
xstr = 'Time index n';
n = [0:N-1];
Nd_lp = (N-1)/2;
Nd_hp = N - Nd_lp - 1;
ystr1 = 'h_{lp}(n)';
ystr2 = 'h_{hp}(n)';
tstr1 = ['Analysis low pass filter, N_{lp} = ',int2str(N),...
         ', delay = ', num2str(Nd_lp), ' samples'];
tstr2 = ['Analysis high pass filter, N_{hp} = ',int2str(N),...
         ', delay = ', num2str(Nd_hp), ' samples'];
subplot(2,1,1), discret(n,h1);
h1 = ylabel(ystr1); set(h1,'fontsize',axfnt);
h1 = title(tstr1); set(h1,'fontsize',axfnt);
set(gca,'fontsize',ticfnt);
subplot(2,1,2), discret(n,h2);
h1 = xlabel(xstr); set(h1,'fontsize',axfnt);
h1 = ylabel(ystr2); set(h1,'fontsize',axfnt);
h1 = title(tstr2); set(h1,'fontsize',axfnt);
set(gca,'fontsize',ticfnt);

pos(1) = pos(1) - 0.495;  % position of next figure
ftf = figure('name','Frequency domain (y-zoom on)','Units','normal','Position',pos);

xstr = 'f normalized to Fsample/2';
ystr = 'Magnitude in dB';
tstr1 = 'QMF-bank transfer functions (linear phase filters)';
tstr2 = ['Overall transfer function, filter bank delay = ',...
         int2str(N-1), ' samples'];
subplot(2,1,1), plot(f,20*log10(magH1),'y',f,20*log10(magH2),'m');
h1 = ylabel(ystr); set(h1,'fontsize',axfnt);
h1 = title(tstr1); set(h1,'fontsize',axfnt); grid on
set(gca,'fontsize',ticfnt);
subplot(2,1,2), plot(fy,20*log10(magY),'y');
h1 = xlabel(xstr); set(h1,'fontsize',axfnt);
h1 = ylabel(ystr); set(h1,'fontsize',axfnt);
h1 = title(tstr2); set(h1,'fontsize',axfnt); grid on
set(gca,'fontsize',ticfnt);
zoom(ftf,'yon');
