function h = qmf_2hb(N,fs,type)
% function h = qmf_2hb(N,fs,type)
%
% design of 2-channel orthonormal QMF-bank, typ. h = qmf_2hb(50,0.633,'f');
% (halfband filter design and spectral factorization using cepstrum)
%
% N            FIR filter length (forced to be even)
% fs           stop-band edge frequency normalized to Fsampl/2 (0.5 < fstop < 1)
%              Note: decrease fs, if remez() diverges because of high stop band att.!
% type         'f'   for FIR filters
%              'l'   for QMF lattice filters
% h            if type = 'f', h is column vector of filter impulse responses
%                             h = [h_lp;h_hp;g_lp;g_hp]
%                             h_lp, h_hp   analysis filter impulse responses
%                             g_lp, g_hp   synthesis filter impulse responses
%              if type = 'l'  h is column vector of lattice filter coefficients
%
% G. Doblinger, TU-Wien, 01-1998
%
% needs extern function Es = QMF_Lat_stop(alpha,fs) for nonlinear optimization

% ref.: Strang, Nguyen, "Wavelets and Filterbanks", p.110, p.163.
%       G.A.Mian, A.P.Nainer, "A Fast Procedure to Design Equiripple Minimum-Phase
%       FIR-Filters, IEEE CAS-29, pp. 327-331, May 1982.

%       P.P.Vaidyanathan, P.Hoang, "Lattice structures for optimal design and robust
%       implementation of two-channel perfect-reconstruction QMF banks", IEEE, Trans.
%       ASSP, Vol. 36, pp.81-94, Jan. 1988.

if nargin == 0
   help qmf_2hb
   return
elseif fs <= 0.5 | fs >= 1
   error('fs must be in (0.5,1)');
end
fg = 2*(1-fs);                                    % pass band edge of full band filter
if rem(N,2)
   N = N + 1;
end

% half band filter is obtained by designing full-band filter
% (with pass band edge = fg) and inserting zeros between
% samples (excluding mid-point)

Nhb = 2*N-1;
h = zeros(Nhb,1);
hg = remez(N-1,[0 fg 1 1],[0.5 0.5 0 0]);         % odd index samples of half band filter
h(1:2:Nhb) = hg(:);
h((Nhb+1)/2) = 0.5;                               % mid-point sample

% spectral factorization to find QMF lowpass filter

Nfft = 2^(ceil(log2(8*Nhb)));               % use FFT length of approx. 8*N
c = cceps(h,Nfft);                          % complex cepstrum of halfband filter coeff.
c1 = c(1:N);
c2 = c(Nfft:-1:Nfft-N+2);
c = 0.5*[c1(1) ; (c1(2:N)+c2)];             % c(n) = 0.5*(c1(n) + c1(-n)) n > 0
                                            % c(0) = 0.5*c1(0)

% compute minimum phase filter from cepstral coefficients

h1 = zeros(N,1);
h1(1) = exp(c(1));
h1(2) = c(2)*h1(1);
for n = 2:N-1                               % recursion to find impulse response
   ni = n:-1:1;
   h1(n+1) = sum(c(ni+1).*ni(:).*h1(1:n))/n;
end
h1 = sqrt(2)*h1;

% create periodic white noise input signal for recostruction error computation

Nx = 1024;
Nx2 = Nx/2;
x = perinois(Nx,Nx2,[],sqrt(2));
x = x(:).';

type = lower(type);
if strcmp(type,'f')  % compute FIR QMF bank filters (orthogonal filters)

   h2 = h1(N:-1:1);
   g1 = h2;                                    % g1(n) = h1(N-n)
   h2(2:2:N) = -h2(2:2:N);                     % h2(n) = (-1)^n * h1(N-n)
   g2 = h1;
   g2(1:2:N) = -g2(1:2:N);                     % g2(n) = (-1)^(n-1) * h1(n)
   h = [h1;h2;g1;g2];

   % check orthonormality

   R_h1h1 = xcorr(h1,h1);
   R_h1h2 = xcorr(h1,h2);
   disp(sprintf('Check of Orthonormality: sum h_lp(k)*h_lp(k) = %1.12e', R_h1h1(N)));
   disp(sprintf('                         sum h_lp(k)*h_lp(k+2) = %1.12e', R_h1h1(N+2)));
   disp(sprintf('                         sum h_lp(k)*h_hp(k) = %1.12e', R_h1h2(N)));
   disp(sprintf('                         sum h_lp(k)*h_hp(k+2) = %1.12e', R_h1h2(N+2)));

   % compute reconstruction error
 
   Y = [filter(h1,1,x) ; filter(h2,1,x)];             % analysis filter bank
   U = zeros(size(Y));
   m = 1:2:Nx;
   U(:,m) = Y(:,m);                                   % down/upsampling
   y = filter(g1,1,U(1,:)) + filter(g2,1,U(2,:));     % synthesis filter bank
   y = y(Nx2+1:Nx);                                   % take second period to avoid filter
                                                      % settling time
else  % compute QMF lattice filter

   alpha = Fir2Lattice(h1);   % starting solution for nonlinear optimization

   % solve nonlinear optimization problem

   vs = version;
   if strcmp(vs(1),'5')
      options(1) = 0;            % turn off display
      options(2) = 1e-5;         % termination criterion for variable alpha
      options(3) = 1e-5;         % termination criterion for stop band cost function
      options(6) = 1;            % DFP-algorithm for fminu()
      options(14) = 5000;        % maximum number of iterations

      disp('performing nonlinear function minimization ...');
      t0 = clock;
      [alpha,opt] = fminu('QMF_Lat_stop',alpha,options,[],fs);
      disp([int2str(opt(10)),' function evaluations']);
      disp(['execution time = ',num2str(etime(clock,t0)),' seconds']);
   else                          % new optimization toolbox with MATLAB 6.0
      options = optimset('Display','off','TolX',1e-6,'TolFun',1e-6,...
         'MaxIter',6000,'LargeScale','off');
      disp('performing nonlinear function minimization ...');
      t0 = clock;
      alpha = fminunc('QMF_Lat_stop',alpha,options,fs);
      disp(['execution time = ',num2str(etime(clock,t0)),' seconds']);
   end
   
   h = alpha;
   x1 = [1 ; zeros(N-1,1)];
   [h1,h2] = QMFaLattice(x1,0,alpha);     % compute analysis filter impuls responses
   [y_lp,y_hp] = QMFaLattice(x,1,alpha);
   y = QMFsLattice(y_lp,y_hp,1,alpha);
   y = y(Nx2+1:Nx);                       % reconstruction error signal
end

% compute and plot transfer function

f = linspace(0,1,Nx2);
fy = linspace(0,1,Nx2/2);

magH1 = abs(freqz(h1,1,pi*f));
magH1 = magH1/max(magH1);
eps1 = 1e-6;
i = find(magH1 <= eps1);
if ~isempty(i)
   magH1(i) = eps1 * ones(size(i));
end
magH2 = abs(freqz(h2,1,pi*f));
magH2 = magH2/max(magH2);
i = find(magH2 <= eps1);
if ~isempty(i)
   magH2(i) = eps1 * ones(size(i));
end
magY = fft(y);
magY = abs(magY(1:Nx2/2));

i = find(magY <= eps1);
if ~isempty(i)
   magY(i) = eps1 * ones(size(i));
end

close all
if strcmp(upper(computer),'PCWIN')
   pos = [0.5 0.5 0.49 0.415];
   axfnt = 8;
   ticfnt = 7;
else	
   pos = [0.5 0.6 0.49 0.4];
   axfnt = 12;
   ticfnt = 10;
end

ftd = figure('name','Time domain','Units','normal','Position',pos);
figurebackcolor = 'black';
colordef(ftd,figurebackcolor);

xstr = 'Time index n';
n = [0:N-1];
Nd_lp = round(sum(n'.*(h1.^2))/sum(h1.^2));   % energy delays
Nd_hp = N - Nd_lp - 1;
ystr1 = 'h_{lp}(n)';
ystr2 = 'h_{hp}(n)';
tstr1 = ['Analysis LP filter, N_{lp} = ',int2str(N),...
         ', energy delay = ', num2str(Nd_lp), ' samples'];
tstr2 = ['Analysis HP filter, N_{hp} = ',int2str(N),...
         ', energy delay = ', num2str(Nd_hp), ' samples'];
subplot(2,1,1), discret(n,h1);
h1 = ylabel(ystr1); set(h1,'fontsize',axfnt);
h1 = title(tstr1); set(h1,'fontsize',axfnt);
set(gca,'fontsize',ticfnt);
subplot(2,1,2), discret(n,h2);
h1 = xlabel(xstr); set(h1,'fontsize',axfnt);
h1 = ylabel(ystr2); set(h1,'fontsize',axfnt);
h1 = title(tstr2); set(h1,'fontsize',axfnt);
set(gca,'fontsize',ticfnt);

pos(1) = pos(1) - 0.495;  % position of next figure
ftf = figure('name','Frequency domain (y-zoom on)','Units','normal','Position',pos);
colordef(ftf,figurebackcolor);

xstr = 'f normalized to Fsample/2';
ystr = 'Magnitude in dB';
if strcmp(type,'l')
   tstr1 = 'QMF-bank transfer functions (lattice filters)';
else
   tstr1 = 'QMF-bank transfer functions (FIR filters)';
end
tstr2 = ['Overall transfer function, filter bank delay = ',...
         int2str(N-1), ' samples'];
subplot(2,1,1), plot(f,20*log10(magH1),'y',f,20*log10(magH2),'m');
h1 = ylabel(ystr); set(h1,'fontsize',axfnt);
h1 = title(tstr1); set(h1,'fontsize',axfnt); grid on
set(gca,'fontsize',ticfnt);
subplot(2,1,2), plot(fy,20*log10(magY),'y');
h1 = xlabel(xstr); set(h1,'fontsize',axfnt);
h1 = ylabel(ystr); set(h1,'fontsize',axfnt);
h1 = title(tstr2); set(h1,'fontsize',axfnt); grid on
set(gca,'fontsize',ticfnt);
zoom(ftf,'yon');

% --------------------------------------------------------------------------------------

function alpha = Fir2Lattice(h)

% compute lattice coefficients of 2 channel QMF lattice filter bank
%
% h      FIR filter impulse response vector (even length N)
% alpha  column vector of lattice coefficients (length N/2)

% Db, 6.1.98

% ref.: P.P.Vaidyanathan, P.Hoang, "Lattice structures for optimal design and robust
%     implementation of two-channel perfect-reconstruction QMF banks", IEEE, Trans.
%     ASSP, Vol. 36, pp.81-94, Jan. 1988.

N = length(h);
if rem(N,1)
   error('Fir2Lattice: filter length must be even');
end

%alpha = zeros(N/2,1);
%p = h;
%m = N/2;
%for n = N-1:-2:2
%   alpha(m) = -p(n+1)/p(1);
%   pt = p(1:n);
%   p = p(n+1-[0:n-1]);
%   p(1:2:n) = -p(1:2:n);
%   p = 1/(1+alpha(m)^2)*(pt + alpha(m)*p)
%   m = m-1;
%end
%alpha(1) = -p(2)/p(1);

alpha = zeros(N,1);
p = h;
q = p(N:-1:1);
q(1:2:N) = -q(1:2:N);
for n = N-1:-1:2
   alpha(n) = -p(n+1)/q(n+1);
   p = 1/(1+alpha(n)^2)*(p(1:n) + alpha(n)*q(1:n));
   q = p(n:-1:1);
   q(1:2:n) = -q(1:2:n);
end
alpha(1) = -p(2)/p(1);
alpha = alpha(1:2:N);

% --------------------------------------------------------------------------------------

function [y_lp,y_hp] = QMFaLattice(x,dec,alpha)
%function [y_lp,y_hp] = QMFaLattice(x,dec,alpha)
%
% compute response of 2-channel QMF lattice analysis filter bank
%
% x            input signal vector
% dec          if 0, then no downsampling is applied
%                 1, downsampling by factor 2
% alpha        vector of lattice coefficients computed with Fir2Lattice()
% y_lp, y_hp   output signal vectors of low pass and high pass filters

% Db, 6.1.98
% ref.: P.P.Vaidyanathan, "Lattice structures for optimal design and robust
%     implementation of two-channel perfect-reconstruction QMF banks", IEEE, Trans.
%     ASSP, Vol. 36, pp.81-94, Jan. 1988.

[Nxr,Nxc] = size(x);
if min(Nxr,Nxc) > 1
   error('QMFaLattice: input signal must not be a matrix');
end
Nx = max(Nxr,Nxc);
N = length(alpha);

% compute lattice input signals

p0 = x(:);
q0 = [0 ; p0(1:Nx-1)];
if dec
   p0 = p0(2:2:Nx);
   q0 = q0(2:2:Nx);
   zpad = 0;
   Nq = length(q0)-1;
else
   zpad = [0 ; 0];
   Nq = Nx-2;
end

% compute first stage separately

p = p0 - alpha(1)*q0;
q = q0 + alpha(1)*p0;

for n = 2:N
   q = [zpad ; q(1:Nq)];   % delayed q-vector
   pt = p;
   p = p - alpha(n)*q;
   q = q + alpha(n)*pt;
end

beta = 1/sqrt(prod(1+alpha.^2));
y_lp = beta*p;
y_hp = beta*q;
if Nxr == 1
   y_lp = y_lp.';
   y_hp = y_hp.';
end

% --------------------------------------------------------------------------------------

function y = QMFsLattice(x_lp,x_hp,int,alpha)
%function y = QMFsLattice(x_lp,x_hp,int,alpha)
%
% compute response of 2-channel QMF lattice synthesis filter bank
%
% x_lp, x_hp   input signal vectors computed with QMFaLattice()
% int          if 0, then no upsampling is applied
%                 1, upsampling by factor 2
% alpha        vector of lattice coefficients computed with Fir2Lattice()
% y            output signal vector

% Db, 6.1.98
% ref.: P.P.Vaidyanathan, P.Hoang, "Lattice structures for optimal design and robust
%     implementation of two-channel perfect-reconstruction QMF banks", IEEE, Trans.
%     ASSP, Vol. 36, pp.81-94, Jan. 1988.

[Nxr,Nxc] = size(x_lp);
if min(Nxr,Nxc) > 1
   error('QMFsLattice: input signal must not be a matrix');
end
Nx = max(Nxr,Nxc);
N = length(alpha);
alpha = alpha(N:-1:1);     % use reversed coefficients

if int
   zpad = 0;
   Nq = Nx-1;
else
   zpad = [0 ; 0];
   Nq = Nx-2;
end

% compute first stage separately

x_lp = x_lp(:);
x_hp = x_hp(:);
p = x_lp + alpha(1)*x_hp;
q = x_hp - alpha(1)*x_lp;

for n = 2:N
   p = [zpad ; p(1:Nq)];   % delayed p-vector
   pt = p;
   p = p + alpha(n)*q;
   q = q - alpha(n)*pt;
end

if int
   Nx = 2*Nx;
   p_int = zeros(Nx,1);
   p_int(2:2:Nx) = p;
   q_int = zeros(Nx,1);
   q_int(2:2:Nx) = q;
   p = 2*p_int;
   q = 2*q_int;
end
p = [0 ; p(1:Nx-1)];

y = (p + q)/(2*sqrt(prod(1+alpha.^2)));

if Nxr == 1
   y = y.';
end