function y = pitch_scale(x,scale,N)
% function  y = pitch_scale(x,scale,N)
%
% pitch scaling of audio signals using FFT filter bank
% x      real-valued input signal
% scale  pitch scale factor in [0.5,2]
% N      time window length
% y      output signal
%
% G. Doblinger, TU-Wien, 02-2001

M = round(N/4);           % decimation factor
h = hanning(N);           % time window function
h = 2*sqrt(M/N)*h(:);
Nf = 2^(1+ceil(log2(N))); % FFT length (power of 2 >= 2*N)
Nh = Nf/2+1;
x = [x(:) ; zeros(N-length(x),1)];  % append zeros, if necessary
nsig = length(x);

% initialize pitch scaling variables

ana_phase = zeros(Nh,1);            % instantaneous phase phi(k)         
freq = 2*pi/Nf*[0:Nh-1]';           % FFT frequency grid theta(k)
c1 = 1/M;
c2 = M;
diffph0 = c2*freq;                  % theta(k)*M
if scale < 1
   nmax = floor(scale*Nh);
   Zpad = zeros(Nh-nmax,1);
else
   nmax = Nh;
   Zpad = [];
end
syn_phase = zeros(nmax,1);          % phi_hat(k) 

ks = round([1:nmax]'/scale);        % resampled frequency indices
   
y = zeros(nsig,1);
for m = 1:M:nsig-N+1                % frame index loop

   % analysis filter bank

   X = fft(x(m:m+N-1).*h,Nf);
   X = X(1:Nh);

   % compute instantaneous phase ana_phase
   
   ph = atan2(imag(X),real(X));
   diffph = ph - ana_phase - diffph0;
   ana_phase = ph;
   
   % limit differential phase to [-pi,pi]
   
   diffph = rem(diffph,2*pi);
   ii = find(diffph > pi);
   diffph(ii) = diffph(ii) - 2*pi;
   ii = find(diffph < -pi);
   diffph(ii) = diffph(ii) + 2*pi;

   % compute instantaneous frequencies
   
   ana_freq = freq + c1*diffph;
   
   % pitch scaling by decimating/interpolating of frequency bins
   
   ana_mag = abs(X);
   syn_mag = ana_mag(ks);
   syn_freq = scale*ana_freq(ks);
   
   % synthesis step, and synthesis filter bank
   
   syn_phase = rem(syn_phase + M*syn_freq,2*pi);
   X = syn_mag .* exp(j*syn_phase);
   X = [X ; Zpad];
   X = [X ; conj(X(Nh-1:-1:2))];

   y1 = real(ifft(X,Nf));
   y(m:m+N-1) = y(m:m+N-1) + y1(1:N).*h;
end
