function dftplot(x,Nf,dBrange)
% function dftplot(x,Nf,dBrange)
%
% call: dftplot(x), dftplot(x,Nf), dftplot(x,Nf,dBrange)
%
% plot signal spectrum, display magnitude (lin. and dB scale),
% phase, and group delay
%
% x         vector of signal from time index n=0 to n=length(x)-1
% Nf        optional number of frequency points (default is 1024)
%           if length(x) < Nf, then zeros are appended to x
%           if length(x) > Nf, then Nf = length(x)
% dBrange   optional dynamic range of dB scale (default is 100 dB)
%
% copyright (c), G. Doblinger, TU Wien, 7-94

% Db, 7-94, 11-2000 (new plot style)

if nargin == 0
   help dftplot1
   return
elseif nargin == 1
   dBrange = 100;
   Nf = 1024;
   Nx = length(x);
   N = max(Nf,Nx);                  % N is number of frequency points used
   N = 2^(ceil(log2(N)));
elseif nargin == 2
   dBrange = 100;
   Nx = length(x);
   N = Nf;
elseif nargin == 3
   Nx = length(x);
   if isempty(Nf)
      Nf = 1024;
      N = max(Nf,Nx);
      N = 2^(ceil(log2(N)));
   else
      N = Nf;
   end
else
   error('DFTPLOT: wrong number of arguments')
end

if min(size(x)) ~= 1
   error('DFTPOLT: x must be a vector');
elseif imag(abs(x)) > eps
   error('DFTPOLT: x must be a real-valued vector');
end

% section to compute spectrum

X = fft(x(:),N);                    % X is FT of x(n)
X1 = fft([0:Nx-1]'.*x(:),N);        % X1 is FT of n*x(n) used to compute
                                    % group delay without differentiation,
                                    % since X1 = j*dX/dw, and
                                    % tg = -imag(d(ln(X)/dw) = real(X1/X)
Nh1 = floor(N/2) + 1;
X = X(1:Nh1);
X1 = X1(1:Nh1);
Xlog = log10(X + 1e-15);            % take complex logarithm
Xmag = 20*real(Xlog);               % compute magnitude in dB

magmax = max(Xmag);
magmin = min(Xmag);
if magmax - magmin < 1e-10          % avoid log rounding errors
                                    % (set resolution to 1e-10 dB)
   Xmag = zeros(size(X));
end
dBrange = abs(dBrange);
if magmax - magmin > dBrange        % eliminate spectral nulls
                                    % (set dynamic range to dBrange dB)
   imin = find(Xmag < magmax-dBrange);
   Xmag(imin) = (magmax-dBrange)*ones(length(imin),1);
end

tgflag = 0;
Xr = real(X);  Xi = imag(X);
if abs(Xr) < 1e-10                  % real part approx. zero
   Xr = zeros(size(X));
   ph = zeros(size(X));
   ip = find(Xi > 1e-10);
   in = find(Xi < -1e-10);
   if ~isempty(ip), ph(ip) = pi/2 * ones(length(ip),1); end
   if ~isempty(in), ph(in) = -pi/2 * ones(length(in),1); end
   tg = zeros(size(X));
elseif abs(Xi) < 1e-10              % imaginary part approx. zero
   Xi = zeros(size(X));
   ph = zeros(size(X));
   tg = zeros(size(X));
else
   ph = 1/log10(exp(1))*imag(Xlog);    % compute phase
   tg = real(X1 ./ (X + 1e-15));       % group delay
   inan = find(abs(tg) >= 1e3);        % eliminate isolated group delay spikes
   if ~isempty(inan)                   % interpolate at spike positions
      Ninan = length(inan);
      tg(inan) = 0.5*(tg(max(1,inan-1))+tg(min(Ninan,inan+1)));
      if Ninan > 1 & inan(Ninan) == Ninan
         tg(Ninan) = tg(Ninan-1);
      end
      if inan(1) == 1, tg(1) = tg(2); end
   end
   if abs(max(tg) - min(tg)) < 1e-3    % nearly constant group delay
      tg = max(tg)*ones(size(tg));
   end
   inan = find(abs(tg) >= 1e10);       % remaining group delay spikes
   if ~isempty(inan)
      tgflag = 1;
      disp('DFTPLOT INFO: Phase/Group delay computation failed');
   else
      tgflag = 0;
   end
end

% section to plot results (comb-like plot for Nf < 40)

close all
figurebackcolor = 'black';

if strcmp(upper(computer),'PCWIN')
   pos1 = [0.5 0.51 0.49 0.405];
   axfnt = 9;
   ticfnt = 8;
   yoff = 0.49;
else	
   pos1 = [0.5 0.6 0.49 0.38];
   axfnt = 13;
   ticfnt = 12;
   yoff = 0.51;
end
fsig = figure('numbertitle','off','name','Time domain',...
   'Units','normal','Position',pos1);
colordef(fsig,figurebackcolor);

t = 0:Nx-1;
if Nx <= 100
   str_plot = 'discret';

else
   str_plot =  'plot';
end
eval([str_plot,'(t,x)']);
h1 = xlabel('Time index');
h2 = ylabel(inputname(1));
h3 = title('Signal waveform');
set(h1,'fontsize',axfnt);
set(h2,'fontsize',axfnt);
set(h3,'fontsize',axfnt);
set(gca,'fontsize',ticfnt);
if strcmp(str_plot,'discret')
   dn = ceil(Nx/70);           % use for offset of x-axis
   xax = axis;
   xax(1) = xax(1) - dn;
   xax(2) = xax(2) + dn;
   axis(xax);
else
   grid on;
end

pos1(1) = pos1(1) - 0.495;  % position of next figure

fmag = figure('numbertitle','off','name',...
   'Frequency domain: DFT magnitude (x-zoom on)',...
   'Units','normal','Position',pos1);
colordef(fmag,figurebackcolor);

f = [linspace(0,1,Nh1)]';
subplot(2,1,1), plot(f,abs(X)), grid on;
h1 = xlabel('Frequency normalized to Fs/2');
h2 = ylabel('Magnitude');
h3 = title(['DFT length = ', num2str(N)]);
set(h1,'fontsize',axfnt);
set(h2,'fontsize',axfnt);
set(h3,'fontsize',axfnt);
set(gca,'fontsize',ticfnt);
subplot(2,1,2), plot(f,Xmag), grid on;
h2 = ylabel('Magnitude in dB');
set(h2,'fontsize',axfnt);
set(gca,'fontsize',ticfnt);
zoom(fmag,'xon');

if ~tgflag
   pos1(2) = pos1(2) - yoff;  % position of next figure
   fph = figure('numbertitle','off','name',...
      'Frequency domain: DFT phase, and group delay (x-zoom on)',...
      'Units','normal','Position',pos1);
   colordef(fph,figurebackcolor);

   subplot(2,1,1), plot(f,ph), grid on;
   h1 = xlabel('Frequency normalized to Fs/2');
   h2 = ylabel('DFT phase (rad)');
   set(h1,'fontsize',axfnt);
   set(h2,'fontsize',axfnt);
   set(gca,'fontsize',ticfnt);
   subplot(2,1,2), plot(f,tg), grid on;
   h2 = ylabel('Group delay (samples)');
   set(h2,'fontsize',axfnt);
   set(gca,'fontsize',ticfnt);
   zoom(fph,'xon');
end
