  function [y,V] = perinois(N,p,mag,normy)
% function [y,V] = perinois(N,p,mag,normy), or [y,V] = perinois(N,p,mag),
% or       [y,V] = perinois(N,p)
%
% generate a real periodic noise sequence with given spectral density
% and approximately gaussian PDF (for large p)
%
% N      number of noise samples (N >= p, otherwise N is set to p)
% p      period duration (forced to nearest power of 2 greater or equal to p)
% mag    vector of dim. p/2 + 1 containing the specified spectral density
%        at frequency points from 1 to p/2 + 1 (half the sampling frequency)
%        (if omitted or empty, than a constant spectral density is used)
% normy  norm of generated noise (1, if omitted)
% y      column vector of dim. N of noise samples
% V      complex spectrum of one noise period (column vector)
%
% Db, 24.10.91, 20.10.92

   if nargin < 2
      help perinois
      return
   end

   % set period duration p to a power of 2

   pmax = 16;           % maximun period is 2^pmax
   pow2 = 2 .^[0:pmax]; % vector of powers of 2      
   if p > 2^pmax
      p = 2^pmax;
   else                 % find nearest power of 2
      i2 = find(pow2 >= p);
      p = pow2(i2(1));
   end       
   
   if N < p
      N = p;
      disp('INFO: number of noise samples N set to period duration p');
   end   
   
   % check arguments

   p2 = p / 2;
   if nargin < 3
      mag = ones(p2+1, 1);
      normy = 1;
   elseif nargin < 4
      if ~isempty(mag) & length(mag) ~= p2 + 1
         error('Vector mag must have dimension p/2 + 1');
      end
      normy = 1;
   end
   
   if isempty(mag)
      mag = ones(p2+1, 1);
   elseif length(mag) ~= p2 + 1   
      error('Vector mag must have dimension p/2 + 1');
   end

   % make magnitude symmetric for real noise sequence
   % mag(k) = mag(p-k+2) k = 2 ... p/2

   mag = mag(:);                 % force mag to column vector   
   mag = [mag ; mag(p2:-1:2)];   % append symmetric part
      
   % generate phase vector uniformly distributed in [-pi,pi]
   % with symmetry properties (to obtain a real noise sequence)
   %     phi(1) = phi(p+1) = 0
   %     phi(p/2+1) = 0   
   %     phi(k) = -phi(p-k+2)      k = 2 ... p/2 
   
   phi = zeros(p, 1);
   phi(2:p2) = 2 * pi * (rand(p2-1, 1) - 0.5);
   phi(p2+2:p) = -phi(p2:-1:2); 
   
   % form Fourier transform (with given magnitude) of periodic noise
   % (one period)
   
   V = mag .* exp(j * phi);
   
   % compute one period of noise sequence (force it to a real sequence)
   
   v = real(ifft(V));
   
   % fill up noise vector by repeating period v

   np = ceil(N / p);   
   y = zeros(np*p, 1);
   pm1 = p - 1;
   for n = 1:p:(np-1)*p+1
      y(n:n+pm1) = v;
   end                
   y = y(1:N);                   % cut noise sequence to required length
   y = abs(normy)/norm(y) * y;   % normalize y to required power   
   
