function tfplot(z,p,k)
% function tfplot(z,p,k)
%
% plot transfer function of digital IIR filter from 0 to Fs/2
% (magnitude in dB, group delay, poles and zeros)
%
% p   vector of poles
% z   vector of zeros
% k   gain factor of cascade form
%
% G. Doblinger, TU-Wien, 8-94

if nargin == 0
   help tfplot
   return
end

if max(length(z),length(p)) > 10
   Nf = 1000;
else
   Nf = 600;
end   
f = linspace(0,0.9999,Nf);
[H,tg] = trafu(k,p,z,pi*f);   
HdB = 20*log10(abs(H) + eps);
dBrange = 200;
magmax = max(HdB);
magmin = min(HdB);

if magmax - magmin > dBrange        % eliminate spectral nulls
                                    % (set dynamic range to dBrange dB)
   imin = find(HdB < magmax-dBrange);
   HdB(imin) = (magmax-dBrange)*ones(length(imin),1);
end

close all
figurebackcolor = 'black';
if strcmp(upper(computer),'PCWIN')
   pos1 = [0.5 0.51 0.49 0.405];
   axfnt = 9;
   ticfnt = 8;
   yoff = 0.49;
else	
   pos1 = [0.5 0.6 0.49 0.38];
   axfnt = 13;
   ticfnt = 12;
   yoff = 0.51;
end

fmag = figure('numbertitle','off','name',...
   'Frequency domain: filter transfer function (y-zoom on)',...
   'Units','normal','Position',pos1);
colordef(fmag,figurebackcolor);

subplot(2,1,1), plot(f,HdB), grid on;
h1 = xlabel('Frequency normalized to Fs/2');
h2 = ylabel('Magnitude in dB');
set(h1,'fontsize',axfnt);
set(h2,'fontsize',axfnt);
set(gca,'fontsize',ticfnt);
subplot(2,1,2), plot(f,tg), grid on;
h2 = ylabel('Group delay (samples)');
set(h2,'fontsize',axfnt);
set(gca,'fontsize',ticfnt);
zoom(fmag,'yon');

pos1(1) = pos1(1) - 0.495;  % position of next figure
fpz = figure('numbertitle','off','name','Z-Plane, poles and zeros',...
   'Units','normal','Position',pos1);
colordef(fpz,figurebackcolor);
zplane(z,p);
set(gca,'fontsize',ticfnt);

% -------------------------------------------------------------------------

function [H,grpd] = trafu(k, p, z, w)
%
% compute digital filter transfer function at frequency points given by w
%
% k      gain factor
% p      vector of filter poles
% z      vector of filter zeros
% w      vector of relative frequency points (w = 2*pi for f = f_sampl)
% H      transfer function vector
% grpd   group delay vector
%
% G. Doblinger, TU-Wien, 01-1991

eps1 = 100 * eps;   
f = exp(sqrt(-1) * w);
nw = length(w);
H = zeros(1, nw);
grpd = zeros(1, nw);
for i = 1:nw
   fi = f(i);
   zt = fi - z;
   pt = fi - p;
   H(i) = k * prod(zt) / prod(pt);
   zt = zt(find(abs(zt) > eps1));   % eliminate singularity (in case of
                                    % zeros on unit circle) for group
                                    % delay computation 
   grpd(i) = -sum(real(fi ./ zt)) + sum(real(fi ./ pt));
end
